'''
Created on 24.06.2011

@author: chris
'''

import sys, random
import pygame
from pygame.locals import *
from pygame.color import *
import pymunk
from SupraBall.supraball import SupraBall, OutOfViewException
from SupraBall.Box import Box, FixBox
import Utils
from Level import Level
from pymunk.vec2d import Vec2d


RENDER_MAGNETIC_FORCES = True

QUIT_GAME = False

def loadLevel(space, boxes):
    l = Level()
    l.loadFromFile('level.json') 

    boxWidth = 40
    
    roofLineY = Utils.getScreenHeight()
    bottomLineY = 0
    posX = 0
    
     
    
    for b in l.bottomBoxes:
        fb = FixBox(space, (boxWidth, b.height), (posX, bottomLineY), b.polarity)
        #print "height: " , b.height, "  X: ", posX
        boxes.append(fb)
        posX += boxWidth
    
    posX = 0
    
    for b in l.topBoxes:
        fb = FixBox(space, (boxWidth, b.height), (posX, roofLineY-b.height), b.polarity)
        boxes.append(fb)
        posX += boxWidth
    
def add_box(space):
    x = random.randint(120,380)
    
    box = Box((42,42),(x, 550))
    space.add(box.body, box)
    return box

def draw_ball(screen, ball):
    """Draw a ball shape"""
    #p = int(ball.body.position.x), 600-int(ball.body.position.y)
    #pygame.draw.rect(screen, THECOLORS["blue"], pygame.Rect(p,(10,10)))
    ball.draw(screen)

def add_Ground(space):
    """Add a inverted L shape with two joints"""
    body = pymunk.Body(pymunk.inf, pymunk.inf) # 1
    body.position = (300,-300)
    ground = pymunk.Segment(body, (-10000, 0), (10000, 0), 6.0) # 2
    ground.elasticity = 0.0
    ground.friction = 0.0
    space.add_static(ground) # 3
    return ground

def draw_lines(screen, lines):
    """Draw the lines"""
    for line in lines:
        body = line.body
        pv1 = body.position + line.a.rotated(body.angle)
        pv2 = body.position + line.b.rotated(body.angle)
        p1 = Utils.to_pygame(pv1 + (0,3))
        p2 = Utils.to_pygame(pv2 + (0,3))
        pygame.draw.lines(screen, THECOLORS["lightgray"], False, [p1,p2], 6)


class Woelkchen(object):
    def __init__(self, position ):
        self.position = position

def spawnNewCloud(position):
    return Woelkchen(position + (Utils.getScreenWidth(), random.randint(70, Utils.getScreenHeight()-70)))

def drawBackGround(woelkchen, position, wolken, screen):
    
    if len(wolken) < 5:
        if not wolken:
            wolken.append(spawnNewCloud(position))
        else:
            if (position.x + Utils.getScreenWidth() - wolken[-1].position.x) > 1.5*woelkchen.get_width():
                wolken.append(spawnNewCloud(position))
        
        
    obsolete = []
    
    for w in wolken:
        if (w.position.x + woelkchen.get_width()) < position.x:
            obsolete.append(w)
           
    for w in obsolete:
        wolken.remove(w) 
        
    
    for w in wolken:
        screen.blit(woelkchen, w.position - position)    
    

def main():
    pygame.init()
    screen = pygame.display.set_mode((Utils.getScreenWidth(),
                                      Utils.getScreenHeight()))
    pygame.display.set_caption("SupraBall")

    clock = pygame.time.Clock()
    running = True
    
    pymunk.init_pymunk()
    space = pymunk.Space()
    space.gravity = (0.0, -300.0)
    
    supraBall = SupraBall(space, (250, Utils.getScreenHeight()/2))
#    top = Groove(space, (-500,400), (1000,400))
#    bottom = Groove(space, (-500,200), (1000,200))
    
#    lines = []  
#    
    boxes = []
    
#    box = Box((200,42), (250,200+21), -1)
#    bottom.add(box.body, (-100,0))
#    bottom.add(box.body, (100,0))
#    box.body.apply_force((-42.0,0.0))
#    boxes.append(box)
#    
#    box = Box((42,42), (700, 400), -1)
#    top.add(box.body, (0,0))
#    box.body.apply_force((-42.0,0.0))
#    boxes.append(box)
#    
#    space.add(boxes[0].body, boxes[0], boxes[1].body, boxes[1])
    
    supraBall.body.apply_impulse((42,0))
    
    frames = 25
    scrollPosition = Vec2d(0,0)
    
    loadLevel(space, boxes)
    
    woelkchen = pygame.image.load("woelkchen.png")
    wolken = []
    backGroundColor = woelkchen.get_at((0,0))

    
    while running:
        #if(supraBall.body.position.x < 0):
            
            
        for event in pygame.event.get():
            if event.type == QUIT:
                running = False
            elif event.type == KEYDOWN:
                if event.key == K_ESCAPE:
                    running = False
                elif event.key == K_q:
                    global QUIT_GAME
                    QUIT_GAME = True
                    running = False
                elif event.key == K_UP:
                    supraBall.k_up = True
                elif event.key == K_DOWN:
                    supraBall.k_down = True
                elif event.key == K_SPACE:
                    supraBall.invField()
            elif event.type == KEYUP:
                if event.key == K_UP:
                    supraBall.k_up = False
                elif event.key == K_DOWN:
                    supraBall.k_down = False

        screen.fill(backGroundColor)
        
#        supraBall.body.reset_forces()
#        
#        for box in boxes:
#            magneticForce = Utils.calculateMagneticForce(box, supraBall)
#            supraBall.body.apply_force(magneticForce)
#            if(RENDER_MAGNETIC_FORCES):
#                pygame.draw.line(screen, THECOLORS["black"], 
#                             Utils.to_pygame(supraBall.body.position),
#                             Utils.to_pygame(supraBall.body.position + 10*magneticForce))
#                
#        
#        pygame.draw.line(screen, pygame.Color(0,255,255),
#                         Utils.to_pygame(supraBall.body.position),
#                         Utils.to_pygame(supraBall.body.position + 10*supraBall.body.force), 3)
        
        supraBall.update()
        scrollPosition = Vec2d(
                max((supraBall.body.position - (250, 0)).x, scrollPosition.x),
                0)
        
        drawBackGround(woelkchen, scrollPosition, wolken, screen)
        
        try:
            supraBall.draw(screen, scrollPosition)
        except OutOfViewException as e:
            return e.args
        
        frames += 1
        if frames == 30:
            frames = 0
            
        
#        top.draw(screen)
#        bottom.draw(screen)
#        draw_lines(screen, lines)
        for o in boxes:
            o.draw(screen, scrollPosition)
        
        space.step(1/50.0)
        
        pygame.display.flip()
        clock.tick(50)
        
while not QUIT_GAME:
    main()        
        
sys.exit()

