'''
Created on 25.06.2011

@author: chris
'''


import json
import random, os

class Box(object):
    def __init__(self):
        self.height = 42
        self.foreFieldStrength = 100
        self.polarity = -1
        self.solid = True # e.g. non solid and polarity 0 would imply gaps 
    
    def setDataFromDict(self, dict):
        self.height = dict['height']
        self.foreFieldStrength = dict['foreFieldStrength']
        self.polarity = dict['polarity']
        self.solid = dict['solid']
                
        

class BoxEncoder(json.JSONEncoder):
    ''' a custom JSON encoder for MyClass objects '''
    def default(self, inputClass):
        if not isinstance (inputClass, Box):
            print 'You cannot use the JSON custom BoxEncoder for a non-Box object.'
            return
        return {
                 'height': inputClass.height,
                 'foreFieldStrength': inputClass.foreFieldStrength,
                 'polarity': inputClass.polarity, 
                 'solid': inputClass.solid}
        

class BoxDecoder(json.JSONDecoder):
    def decode (self, json_string):
        # use json's generic decode capability to parse the serialized string
        # into a python dictionary.
        my_class_dict = json.loads(json_string)
        b = Box()
        return b.setDataFromDict(my_class_dict)


                

class LevelEncoder(json.JSONEncoder):
    ''' a custom JSON encoder for MyClass objects '''
    def default(self, inputClass):
        if not isinstance (inputClass, Level):
            print 'You cannot use the JSON custom LevelEncoder for a non-Level object.'
            return
        
        bEnc = BoxEncoder()
        
        topBoxes = []
        bottomBoxes = []
        
        for b in inputClass.topBoxes:
            topBoxes.append(bEnc.default(b))
        
        for b in inputClass.bottomBoxes:
            bottomBoxes.append(bEnc.default(b))
        
        return { 'topBoxes': topBoxes,
                 'bottomBoxes': bottomBoxes}            
        
class Level(object):
    Encoder = LevelEncoder()
    
    def __init__(self):
        self.topBoxes = []
        self.bottomBoxes = []
    
    def saveToFile(self, fileName):
        #with open(fileName, mode='w', encoding='utf-8') as f:
        with open("level.json", mode='w') as f:
            json.dump(self.Encoder.default(self), f, indent=2, encoding='utf-8')
    
    def loadFromFile(self, fileName):
        with open(fileName, 'r') as f:
            allBoxes = json.load(f, encoding='utf-8')
            
            for dict in allBoxes['topBoxes']:
                b = Box()
                b.setDataFromDict(dict)
                self.topBoxes.append(b)
                
            for dict in allBoxes['bottomBoxes']:
                b = Box()
                b.setDataFromDict(dict)
                self.bottomBoxes.append(b)
                
                        
def test():
    print "main"
    
    print "cwd"
    print os.getcwd()
    
    b = Box()
    lEnc = LevelEncoder()
    
    
    l = Level()
    for i in range (7):
        b = Box()
        b.height = random.randint(1, 256)
        l.topBoxes.append(b)
        b = Box()
        b.height = random.randint(1, 256)
        l.bottomBoxes.append(b)


    l2 = Level()
    l2.loadFromFile('level.json')
    
    print "top boxes"
    for b in l2.topBoxes:
        print b.height
    
    print "bottom boxes"
    for b in l2.bottomBoxes:
        print b.height
    
    
    """
    with open("level.json", mode='w') as f:
            json.dump(lEnc.default(l), f, indent=2, encoding='utf-8')
    """     
    #l.saveToFile("test.json")
    

def generate_Level():
    l = Level()
    for i in range(30):
        generateBoxesOnLevel(l.topBoxes, random.randint(1,10) * 15, random.randint(3,7), random.randint(-1,1))
        generateBoxesOnLevel(l.bottomBoxes, random.randint(1,10) * 15, random.randint(3,7), random.randint(-1,1))
    l.saveToFile('level.json')
        
    
 
def generateBoxesOnLevel(line, height, nbrOfBoxes, polarity):    
    for i in range(nbrOfBoxes):
        b = Box()
        b.height = height
        b.polarity = polarity
        line.append(b)
        

generate_Level()            
