'''
Created on 24.06.2011

@author: neo
'''

import pygame
import pymunk
import Utils
from SupraBall import Box
from pymunk import vec2d
from pymunk.vec2d import Vec2d

class SupraBall (pymunk.Circle):
    '''The shape the player manipulates'''
    
    RENDER_MAGNETIC_FORCES = True
    
    def __init__(self, space, position):
        self.mass = 1
        radius = 14
        self.fieldStrength = 100
        self.polarity = -1
        self.fieldIncrement = 0
        self.k_up = False
        self.k_down = False
        self.space = space
        
        inertia = pymunk.moment_for_circle(self.mass, 0, radius)
        body = pymunk.Body(self.mass, inertia)
        body.position = position
        pymunk.Circle.__init__(self, body, radius)
        self.friction = .42
        self.space.add(body, self)
        
        self.magnet_radius = pymunk.Circle(body, 300)
        self.magnet_radius.collision_type = id(self)
        
        self.magnet_objects = set()
        def collision_begin(space, arbiter):
            if arbiter.shapes[0] == self.magnet_radius:
                other = arbiter.shapes[1]
            else:
                other = arbiter.shapes[0]
            
            self.magnet_objects.add(other)
            return False
        
        def collision_separate(space, arbiter):
            if arbiter.shapes[0] == self.magnet_radius:
                other = arbiter.shapes[1]
            else:
                other = arbiter.shapes[0]
            
            self.magnet_objects.discard(other)
            return False
        
        self.space.add_collision_handler(id(self), Box.COLLISION_ID,
                collision_begin, None, None, collision_separate)
        self.space.add(self.magnet_radius)
        
        if self.RENDER_MAGNETIC_FORCES:
            self.forces = []
    
    def invField(self):
        self.polarity =  self.polarity * -1
    
    def draw(self, surface, scrollPosition):
        viewPosition = self.body.position - scrollPosition
        p = Utils.to_pygame(viewPosition)
        epsilon = 10
        if p[0] < 0 or p[1] > 600 + epsilon:
            raise OutOfViewException(scrollPosition.x)
        
        pygame.draw.circle(surface, Utils.colors[self.polarity], p, int(self.radius))
        pygame.draw.circle(surface, Utils.colors[self.polarity], p, int(self.fieldStrength), 2)
        
        if self.RENDER_MAGNETIC_FORCES:
            for force in self.forces:
                pygame.draw.line(surface, (0,0,0),
                        p,
                        Utils.to_pygame(viewPosition + 10*force))

    def update(self):
        if self.k_up and (not self.k_down):
            if self.fieldIncrement == 0:
                self.fieldIncrement = 1
        elif (not self.k_up) and self.k_down:
            if self.fieldIncrement == 0:
                self.fieldIncrement = -1
        else:
            self.fieldIncrement = 0
        
        if self.fieldIncrement != 0:
            self.fieldStrength = max(self.fieldStrength + self.fieldIncrement, 2)
            self.fieldIncrement *= 1.2
        
        self.body.reset_forces()
        if self.RENDER_MAGNETIC_FORCES:
            self.forces = []
        for shape in self.magnet_objects:
            self.apply_magnetic_object(shape)
        
        # constant reinforcement
        self.body.apply_force((200,0))
    
    def apply_magnetic_object(self, shape):
        bb = shape.cache_bb()
        for x in range(int(bb.left), int(bb.right)+1, 10):
            for y in range(int(bb.bottom), int(bb.top)+1, 10):
                point = Vec2d(x,y)
                distance = point - self.body.position
                distanceLenSqr = distance.get_length_sqrd()
                strength = (1/distanceLenSqr) * self.fieldStrength * \
                    shape.fieldStrength * (-1*(self.polarity*shape.polarity))
                force = distance.normalized() * strength
                self.body.apply_force(force)
                if self.RENDER_MAGNETIC_FORCES:
                    self.forces.append(force)

class OutOfViewException(Exception):
    pass

